################################################################################
# Communicates with xend
#
# Copyright 2006-2007 Novell, Inc.
# Author: Charles Coffing <ccoffing@novell.com>
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
#
# This is the file that communicates with the xen hypervisor.
#
################################################################################

import time
try:
    from xen.xend import XendOptions
except:
    pass
from .hypervisor import Hypervisor
from .exceptions import ParamError, XenError
import os

try:
    # Xen 3.1+
    from xen.util.xmlrpcclient import ServerProxy
except:
    try:
        # Xen 3.0.4 and older
        from xen.util.xmlrpclib2 import ServerProxy
    except:
        pass

def parse_string(uri):
    if not uri.startswith("xen") and not uri.startswith("qemu"):
        raise ParamError(err=ParamError.E_INVALID, details=uri)
    return Xend(uri)

################################################################################
# Class: XenHypervisor
#
# Xen Hypervisor specifics.
################################################################################
class Xend(Hypervisor):
    """Thin abstraction to talk to Xend.

    DEPRECATED.  Migrate to libvirt instead.
    """
    XML_RPC_SOCKET = "/var/run/xend/xmlrpc.sock"

    #---------------------------------------------------------------------------
    # Method: __init__
    #
    # Comments:
    #   Sets the connection information to talk to xen.
    #
    #---------------------------------------------------------------------------
    def __init__(self, uri):
        if os.getuid() != 0:
            raise XenError(err=XenError.E_ROOT, details=uri)
        self.uri = 'httpu://%s' % Xend.XML_RPC_SOCKET
        self.server = ServerProxy(self.uri)
        self.use_libvirt = False
        self.virDom = None


    #---------------------------------------------------------------------------
    # Method: delete
    #---------------------------------------------------------------------------
    def delete(self, domid):
        """Destroys a running instance; erases from xend"""
        try:
            self.server.xend.domain.delete(domid)
        except:
            pass


    #---------------------------------------------------------------------------
    # Method: destroy
    #---------------------------------------------------------------------------
    def destroy(self, domid):
        """Destroys a running instance; does not erase from xend"""
        try:
            self.server.xend.domain.destroy(domid)
        except:
            pass


    #---------------------------------------------------------------------------
    # Method: domain
    #---------------------------------------------------------------------------
    def domain(self, domid):
        try:
            return self.server.xend.domain(domid, False)
        except:
            return None



    #---------------------------------------------------------------------------
    # Method: get_dom0_min_memory
    #---------------------------------------------------------------------------
    def get_dom0_min_memory(self):
        return XendOptions.instance().get_dom0_min_mem()


    #---------------------------------------------------------------------------
    # Method: domain_running
    #---------------------------------------------------------------------------
    def domain_running(self, domid):
        domain = self.domain(domid)
        if not domain:
            return False
        for d in domain:
            if type(d) == list and len(d) > 1 and d[0] == 'state':
                # Any state means there is an instance...
                return len(d[1]) > 1
        return False

    #---------------------------------------------------------------------------
    # Method: info
    #---------------------------------------------------------------------------
    def info(self):
        return dict(self.server.xend.node.info())


    #---------------------------------------------------------------------------
    # Method: lookup
    #---------------------------------------------------------------------------
    def lookup(self, domid):
        """Lookup the domain by name, UUID, or hypervisor ID"""
        return self.server.xend.domain(domid, False)


    #---------------------------------------------------------------------------
    # Method: wait_domain
    #---------------------------------------------------------------------------
    def wait_domain(self, domid):
        #  This is ugly (polling instead of event driven) but XenAPI doesn't
        #  yet grok unix socket.
        #
        #  If domain and 'state' are both found, then assume VM is trying
        #  to come up, so keep waiting.  Otherwise, fail.
        for _ in range(10):
            domain = self.domain(domid)
            if not domain:
                return False
            wait = False
            for d in domain:
                if type(d) == list and len(d) > 1 and d[0] == 'state':
                    if len(d[1]) > 1 and not 'p' in d[1]:
                        return True
                    time.sleep(1.0)
                    wait = True
                    break
            if not wait:
                break
        return False

